@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: Pctest.h,v 1.21 2001/01/19 22:18:44 bmah Exp $
+// $Id: Pctest.h,v 1.22 2001/02/01 21:38:41 bmah Exp $
 //
 // Pctest.h
 // Bruce A. Mah <bmah@acm.org>
@@ -96,7 +96,7 @@ class Pctest {
 
     // Perform a test and return statistics
     virtual int Test(TestRecord &tr) = 0;
-    virtual int GetMinSize() = 0;
+    virtual unsigned int GetMinSize() = 0;
 
   protected:
     int initialized;		// initialization flag
