@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4File.cc,v 1.10 2001/01/25 00:03:49 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4File.cc,v 1.12 2001/02/07 19:35:02 bmah Exp $";
 //
-// $Id: PctestIpv4File.cc,v 1.10 2001/01/25 00:03:49 bmah Exp $
+// $Id: PctestIpv4File.cc,v 1.12 2001/02/07 19:35:02 bmah Exp $
 //
 // PctestIpv4File.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -61,10 +61,33 @@ int PctestIpv4File::GetSocketIn() {
 
 }
 
+//
+// PctestIpv4File::SetOriginName
+//
+// Input:  ignored
+//
+// Output:  success code (negative if an error)
+//
+// This method exists primarily to override PctestIpv4::SetOriginName
+// to prevent it from overwriting the originAddress and originName
+// members, which were set when the trace file was read using 
+// PctestIpv4File::SetTargetName.  All we need to do here is a 
+// name lookup.
+//
+int PctestIpv4File::SetOriginName(char *t) {
+    struct hostent *host;	// resolver hostname entry
+
+    originName = strdup(GetName((char *) &originAddress));
+    if (originName == NULL) {
+	fprintf(stderr, "Couldn't allocate memory for origin hostname.\n");
+	return -1;
+    }
+}
+
 //
 // PctestIpv4File::SetTargetName
 //
-// Input:  none
+// Input:  ignored
 //
 // Output:  success code (negative if error)
 //
@@ -84,7 +107,6 @@ int PctestIpv4File::SetTargetName(char *t)
     extern char *ReadFilename;	// user-supplied filename
     extern unsigned int Repetitions; // number of repetitions per packet size
     extern unsigned int StartHop; // starting hop number
-    extern char *TargetHost;	// probe target
     extern int VerboseFlag;	// -v from command-line
 
     bool done = false;		// done reading?
@@ -292,7 +314,7 @@ int PctestIpv4File::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv4File::GetMinSize() 
+unsigned int PctestIpv4File::GetMinSize() 
 {
     return (minsize);
 }
