@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Icmp.cc,v 1.13 2001/01/26 17:21:11 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Icmp.cc,v 1.18 2001/04/03 19:41:35 bmah Exp $";
 //
-// $Id: PctestIpv4Icmp.cc,v 1.13 2001/01/26 17:21:11 bmah Exp $
+// $Id: PctestIpv4Icmp.cc,v 1.18 2001/04/03 19:41:35 bmah Exp $
 //
 // PctestIpv4Icmp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -68,7 +68,7 @@ int PctestIpv4Icmp::GetSocketOut() {
 //
 // Output:
 //
-// A negative icmpCode indicates a timeout.
+// A negative icmpCode indicates an error.
 //
 int PctestIpv4Icmp::Test(TestRecord &tr)
 {
@@ -78,6 +78,10 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
     fd_set readFds;		// reading file descriptors
     int done = 0;
 
+    // Parameters stored as globals
+    extern unsigned int Tos;
+    extern int Timeout;
+
     // If the requested sending size is too small or too large, 
     // then return an error.  The caller should have figured out the 
     // minimum sending size by calling Pctest::GetMinSize().
@@ -89,7 +93,6 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
     // See Section 25.3 of UNIX Network Programming, Second Edition,
     // for why we need to twiddle the byte-orders of some fields but
     // not others, depending on what OS we run.
-    extern unsigned int Tos;
     struct ip ipHeader;
     memset(&ipHeader, 0, sizeof(ipHeader));
 #ifdef __osf__
@@ -157,8 +160,16 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 				 ICMP_MINLEN + icmpPayloadSize);
     ((icmp *)(ipPacket + sizeof(ipHeader)))->icmp_cksum = checksum;
 
+    // Use malloc(3) to allocate (memory-aligned) space for the inbound
+    // packet.
+    char *icmpPacket;
+    icmpPacket = (char *) malloc(IP_MAXPACKET);
+    if (icmpPacket == NULL) {
+	fprintf(stderr, "Couldn't allocate space for inbound packet\n");
+	return -1;
+    }
+
     // Set timeout value and socket select parameters
-    extern int Timeout;
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
@@ -171,11 +182,9 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
     rc = sendto(socketOut, ipPacket, ipPacketSize, 0,
 		(struct sockaddr *) &targetSocketAddress,
 		sizeof(struct sockaddr_in));
-    delete [] icmpPayload;
-    delete [] ipPacket;
     if (rc < 0) {
 	perror("sendto");
-	return rc;
+	goto exittest;
     }
 
     // We need to check the socket until we get a valid packet.
@@ -186,7 +195,7 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
 	    perror("select");
-	    return rc;
+	    goto exittest;
 	}
 
 	// Timestamp after and update test record timestamp fields
@@ -202,13 +211,10 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
-	    const int icmpPacketSize = 1024 + tr.size;
-	    char icmpPacket[IP_MAXPACKET];
-	
-	    rc = read(socketIn, icmpPacket, icmpPacketSize);
+	    rc = read(socketIn, icmpPacket, IP_MAXPACKET);
 	    if (rc < 0) {
 		perror("read");
-		return rc;
+		goto exittest;
 	    }
 	    tr.replsize = rc;
 
@@ -230,7 +236,8 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 	    ipHeaderIn = (ip *) icmpPacket;
 	    if (ipHeaderIn->ip_p != proto) {
 		IF_DEBUG(0, fprintf(stderr, "Received unknown protocol %d in (supposedly) icmp packet\n", ipHeaderIn->ip_p));
-		return -1;
+		rc = -1;
+		goto exittest;
 	    }
 
 #ifdef __osf__
@@ -340,7 +347,13 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 	}
     } while (!done);
 
-    return 0;
+    rc = 0;
+
+  exittest:
+    delete [] icmpPayload;
+    delete [] ipPacket;
+    free(icmpPacket);
+    return rc;
 
 }
 
@@ -352,7 +365,7 @@ int PctestIpv4Icmp::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv4Icmp::GetMinSize() 
+unsigned int PctestIpv4Icmp::GetMinSize() 
 {
     return (sizeof(ip) + ICMP_MINLEN + 4); // need to hardcode size of an ICMP
 				// echo request packet, because the associated
