@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.17 2001/01/26 17:21:12 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.18 2001/02/16 19:35:29 bmah Exp $";
 //
-// $Id: PctestIpv4Raw.cc,v 1.17 2001/01/26 17:21:12 bmah Exp $
+// $Id: PctestIpv4Raw.cc,v 1.18 2001/02/16 19:35:29 bmah Exp $
 //
 // PctestIpv4Raw.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -83,7 +83,7 @@ int PctestIpv4Raw::GetSocketOut() {
 //
 // Output:
 //
-// A negative icmpCode indicates a timeout.
+// A negative icmpCode indicates an error.
 //
 int PctestIpv4Raw::Test(TestRecord &tr)
 {
@@ -94,6 +94,10 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     fd_set readFds;		// reading file descriptors
     int done = 0;
 
+    // Parameters stored as globals
+    extern unsigned int Tos;
+    extern int Timeout;
+
     // If the requested sending size is too small or too large, 
     // then return an error.  The caller should have figured out the 
     // minimum sending size by calling Pctest::GetMinSize().
@@ -105,7 +109,6 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     // See Section 25.3 of UNIX Network Programming, Second Edition,
     // for why we need to twiddle the byte-orders of some fields but
     // not others, depending on what OS we run.
-    extern unsigned int Tos;
     struct ip ipHeader;
     memset(&ipHeader, 0, sizeof(ipHeader));
 #ifdef __osf__
@@ -185,8 +188,16 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     }
     ((udphdr *)(ipPacket + sizeof(ipHeader)))->uh_sum = ~checksum;
 
+    // Use malloc(3) to allocate (memory-aligned) space for the inbound
+    // packet.
+    char *icmpPacket;
+    icmpPacket = (char *) malloc(IP_MAXPACKET);
+    if (icmpPacket == NULL) {
+	fprintf(stderr, "Couldn't allocate space for inbound packet\n");
+	return -1;
+    }
+
     // Set timeout value and socket select parameters
-    extern int Timeout;
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
@@ -199,11 +210,9 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     rc = sendto(socketOut, ipPacket, ipPacketSize, 0,
 		(struct sockaddr *) &targetSocketAddress,
 		sizeof(struct sockaddr_in));
-    delete [] udpPayload;
-    delete [] ipPacket;
     if (rc < 0) {
 	perror("sendto");
-	return rc;
+	goto exittest;
     }
 
     // We need to check the socket until we get a valid packet.
@@ -214,7 +223,7 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
 	    perror("select");
-	    return rc;
+	    goto exittest;
 	}
 
 	// Timestamp after and update test record timestamp fields
@@ -230,13 +239,10 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
-	    const int icmpPacketSize = 1024 + tr.size;
-	    char icmpPacket[IP_MAXPACKET];
-	
-	    rc = read(socketIn, icmpPacket, icmpPacketSize);
+	    rc = read(socketIn, icmpPacket, IP_MAXPACKET);
 	    if (rc < 0) {
 		perror("read");
-		return rc;
+		goto exittest;
 	    }
 	    tr.replsize = rc;
 
@@ -259,7 +265,8 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	    ipHeaderIn = (ip *) icmpPacket;
 	    if (ipHeaderIn->ip_p != proto) {
 		IF_DEBUG(0, fprintf(stderr, "Received unknown protocol %d in (supposedly) ICMP packet\n", ipHeaderIn->ip_p));
-		return -1;
+		rc = -1;
+		goto exittest;
 	    }
 
 #ifdef __osf__
@@ -352,7 +359,13 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	}
     } while (!done);
 
-    return 0;
+    rc = 0;
+
+  exittest:
+    delete [] udpPayload;
+    delete [] ipPacket;
+    free(icmpPacket);
+    return rc;
 
 }
 
