@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4Tcp.h,v 1.3 2001/01/02 18:46:06 bmah Exp $
+// $Id: PctestIpv4Tcp.h,v 1.4 2001/02/01 21:38:49 bmah Exp $
 //
 // PctestIpv4Tcp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -52,7 +52,7 @@ class PctestIpv4Tcp : public PctestIpv4 {
 
     virtual int GetSocketOut();
     virtual int Test(TestRecord &tr);
-    virtual int GetMinSize();
+    virtual unsigned int GetMinSize();
     virtual PctestActionType GetAction(int icmpType, int icmpCode);
    
   protected:
