@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.17 2001/01/26 17:21:14 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.19 2001/02/16 19:35:29 bmah Exp $";
 //
-// $Id: PctestIpv4Udp.cc,v 1.17 2001/01/26 17:21:14 bmah Exp $
+// $Id: PctestIpv4Udp.cc,v 1.19 2001/02/16 19:35:29 bmah Exp $
 //
 // PctestIpv4Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -90,7 +90,7 @@ int PctestIpv4Udp::GetSocketOut() {
 //
 // Output:
 //
-// A negative icmpCode indicates a timeout.
+// A negative icmpCode indicates an error.
 //
 int PctestIpv4Udp::Test(TestRecord &tr)
 {
@@ -101,6 +101,10 @@ int PctestIpv4Udp::Test(TestRecord &tr)
     fd_set readFds;		// reading file descriptors
     int done = 0;
 
+    // Parameters stored as globals
+    extern unsigned int Tos;
+    extern int Timeout;
+
     // If the requested sending size is too small, then return an
     // error.  The caller should have figured out the minimum sending
     // size by calling Pctest::GetMinSize().
@@ -128,15 +132,22 @@ int PctestIpv4Udp::Test(TestRecord &tr)
     }
 
     // Set TOS bits
-    extern unsigned int Tos;
     rc = setsockopt(socketOut, IPPROTO_IP, IP_TOS, (char *) &Tos, sizeof(Tos));
     if (rc < 0) {
 	perror("setsockopt(IP_TOS)");
 	return rc;
     }
 
+    // Use malloc(3) to allocate (memory-aligned) space for the inbound
+    // packet.
+    char *icmpPacket;
+    icmpPacket = (char *) malloc(IP_MAXPACKET);
+    if (icmpPacket == NULL) {
+	fprintf(stderr, "Couldn't allocate space for inbound packet\n");
+	return -1;
+    }
+
     // Set timeout value and socket select parameters
-    extern int Timeout;
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
@@ -149,10 +160,9 @@ int PctestIpv4Udp::Test(TestRecord &tr)
     rc = sendto(socketOut, udpPayload, udpPayloadSize, 0,
 		(struct sockaddr *) &targetSocketAddress,
 		sizeof(struct sockaddr_in));
-    delete [] udpPayload;
     if (rc < 0) {
 	perror("sendto");
-	return rc;
+	goto exittest;
     }
 
     // We need to check the socket until we get a valid packet.
@@ -163,7 +173,7 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
 	    perror("select");
-	    return rc;
+	    goto exittest;
 	}
 
 	// Timestamp after and update test record timestamp fields
@@ -179,13 +189,10 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
-	    const int icmpPacketSize = 1024 + tr.size;
-	    char icmpPacket[IP_MAXPACKET];
-	    
-	    rc = read(socketIn, icmpPacket, icmpPacketSize);
+	    rc = read(socketIn, icmpPacket, IP_MAXPACKET);
 	    if (rc < 0) {
 		perror("read");
-		return rc;
+		goto exittest;
 	    }
 	    tr.replsize = rc;
 
@@ -208,7 +215,8 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	    ipHeader = (ip *) icmpPacket;
 	    if (ipHeader->ip_p != proto) {
 		IF_DEBUG(0, fprintf(stderr, "Received unknown protocol %d in (supposedly) ICMP packet\n", ipHeader->ip_p));
-		return -1;
+		rc = -1;
+		goto exittest;
 	    }
 
 #ifdef __osf__
@@ -301,7 +309,12 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	}
     } while (!done);
 
-    return 0;
+    rc = 0;
+
+  exittest:
+    delete [] udpPayload;
+    free(icmpPacket);
+    return rc;
 
 }
 
@@ -313,7 +326,7 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv4Udp::GetMinSize() 
+unsigned int PctestIpv4Udp::GetMinSize() 
 {
     return (sizeof(ip) + sizeof(udphdr) + 4);
 }
