@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4Udp.h,v 1.8 2001/01/02 18:46:08 bmah Exp $
+// $Id: PctestIpv4Udp.h,v 1.9 2001/02/01 21:38:51 bmah Exp $
 //
 // PctestIpv4Udp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -52,7 +52,7 @@ class PctestIpv4Udp : public PctestIpv4 {
 
     virtual int GetSocketOut();
     virtual int Test(TestRecord &tr);
-    virtual int GetMinSize();
+    virtual unsigned int GetMinSize();
     virtual PctestActionType GetAction(int icmpType, int icmpCode);
    
   protected:
