@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv6File.cc,v 1.9 2001/01/25 00:03:50 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6File.cc,v 1.11 2001/02/07 19:35:02 bmah Exp $";
 //
-// $Id: PctestIpv6File.cc,v 1.9 2001/01/25 00:03:50 bmah Exp $
+// $Id: PctestIpv6File.cc,v 1.11 2001/02/07 19:35:02 bmah Exp $
 //
 // PctestIpv6File.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -61,6 +61,34 @@ int PctestIpv6File::GetSocketIn() {
 
 }
 
+//
+// PctestIpv6File::SetOriginName
+//
+// Input:  ignored
+//
+// Output:  success code (negative if an error)
+//
+// This method exists primarily to override PctestIpv6::SetOriginName
+// to prevent it from overwriting the originAddress and originName
+// members, which were set when the trace file was read using 
+// PctestIpv6File::SetTargetName.  All we need to do here is a 
+// name lookup.
+//
+// Note:  This code comes from PctestIpv4File::SetOriginName.
+// If we can ever do a multiple inheritance on the file-reading
+// test methods, this is a candidate for inclusion in the base
+// file-reading class.
+//
+int PctestIpv6File::SetOriginName(char *t) {
+    struct hostent *host;	// resolver hostname entry
+
+    originName = strdup(GetName((char *) &originAddress));
+    if (originName == NULL) {
+	fprintf(stderr, "Couldn't allocate memory for origin hostname.\n");
+	return -1;
+    }
+}
+
 //
 // PctestIpv6File::SetTargetName
 //
@@ -84,7 +112,6 @@ int PctestIpv6File::SetTargetName(char *t)
     extern char *ReadFilename;	// user-supplied filename
     extern unsigned int Repetitions; // number of repetitions per packet size
     extern unsigned int StartHop; // starting hop number
-    extern char *TargetHost;	// probe target
     extern int VerboseFlag;	// -v from command-line
 
     bool done = false;		// done reading?
@@ -292,7 +319,7 @@ int PctestIpv6File::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv6File::GetMinSize() 
+unsigned int PctestIpv6File::GetMinSize() 
 {
     return (minsize);
 }
