@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6File.h,v 1.5 2001/01/02 18:46:13 bmah Exp $
+// $Id: PctestIpv6File.h,v 1.7 2001/02/07 19:35:02 bmah Exp $
 //
 // PctestIpv6File.h
 // Bruce A. Mah <bmah@acm.org>
@@ -56,11 +56,12 @@ class PctestIpv6File : public PctestIpv6 {
 	}
     };
 
+    virtual int SetOriginName(char *t);
     virtual int SetTargetName(char *t);
     virtual int GetSocketOut();
     virtual int GetSocketIn();
     virtual int Test(TestRecord &tr);
-    virtual int GetMinSize();
+    virtual unsigned int GetMinSize();
    
   protected:
 
