@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv6Icmp.cc,v 1.5 2001/01/27 00:28:44 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6Icmp.cc,v 1.7 2001/02/16 19:35:29 bmah Exp $";
 //
-// $Id: PctestIpv6Icmp.cc,v 1.5 2001/01/27 00:28:44 bmah Exp $
+// $Id: PctestIpv6Icmp.cc,v 1.7 2001/02/16 19:35:29 bmah Exp $
 //
 // PctestIpv6Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -90,7 +90,7 @@ int PctestIpv6Icmp::GetSocketOut() {
 //
 // Output:
 //
-// A negative icmpCode indicates a timeout.
+// A negative icmpCode indicates an error.
 //
 int PctestIpv6Icmp::Test(TestRecord &tr)
 {
@@ -101,6 +101,9 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
     fd_set readFds;		// reading file descriptors
     int done = 0;
 
+    // Parameters stored as globals
+    extern int Timeout;
+
     // If the requested sending size is too small, then return an
     // error.  The caller should have figured out the minimum sending
     // size by calling Pctest::GetMinSize().
@@ -146,8 +149,16 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 	return rc;
     }
 
+    // Use malloc(3) to allocate (memory-aligned) space for the inbound
+    // packet.
+    char *icmp6PacketIn;
+    icmp6PacketIn = (char *) malloc(IPV6_MAXPACKET);
+    if (icmp6PacketIn == NULL) {
+	fprintf(stderr, "Couldn't allocate space for inbound packet\n");
+	return -1;
+    }
+
     // Set timeout value and socket select parameters
-    extern int Timeout;
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
@@ -160,10 +171,9 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
     rc = sendto(socketOut, icmp6Packet, icmp6PacketSize, 0,
 		(struct sockaddr *) &targetSocketAddress,
 		sizeof(struct sockaddr_in6));
-    delete [] icmp6Payload;
     if (rc < 0) {
 	perror("sendto");
-	return rc;
+	goto exittest;
     }
 
     // We need to check the socket until we get a valid packet.
@@ -174,7 +184,7 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
 	    perror("select");
-	    return rc;
+	    goto exittest;
 	}
 
 	// Timestamp after and update test record timestamp fields
@@ -190,12 +200,9 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
-	    const int icmp6PacketInSize = 1500 + tr.size;
-	    char icmp6PacketIn[IPV6_MAXPACKET];
 	    struct msghdr msg;	// msghdr is for recvmsg
 	    struct iovec iov[1];
 	    int controlsize;
-	    struct cmsghdr *cmsgptr;
 
 	    // Fill in the message header so we can read all the
 	    // metadata from the ICMP packet.  A lot harder than
@@ -203,7 +210,7 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 	    msg.msg_name = (char *) &icmpSourceSocketAddress;
 	    msg.msg_namelen = sizeof(icmpSourceSocketAddress);
 	    iov[0].iov_base = icmp6PacketIn;
-	    iov[0].iov_len = icmp6PacketInSize;
+	    iov[0].iov_len = IPV6_MAXPACKET;
 	    msg.msg_iov = iov;
 	    msg.msg_iovlen = 1;
 
@@ -227,7 +234,7 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 	    rc = recvmsg(socketIn, &msg, 0);
 	    if (rc < 0) {
 		perror("read");
-		return rc;
+		goto exittest;
 	    }
 	    tr.replsize = rc;
 
@@ -351,7 +358,13 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 
     } while (!done);
 
-    return 0;
+    rc = 0;
+
+  exittest:
+    delete [] icmp6Payload;
+    delete [] icmp6Packet;
+    free(icmp6PacketIn);
+    return rc;
 
 }
 
@@ -363,7 +376,7 @@ int PctestIpv6Icmp::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv6Icmp::GetMinSize() 
+unsigned int PctestIpv6Icmp::GetMinSize() 
 {
     return (sizeof(ip6_hdr) + sizeof(icmp6_hdr) + 4);
 }
