@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv6Udp.cc,v 1.26 2001/01/27 00:28:45 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6Udp.cc,v 1.28 2001/02/16 19:35:29 bmah Exp $";
 //
-// $Id: PctestIpv6Udp.cc,v 1.26 2001/01/27 00:28:45 bmah Exp $
+// $Id: PctestIpv6Udp.cc,v 1.28 2001/02/16 19:35:29 bmah Exp $
 //
 // PctestIpv6Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -104,7 +104,7 @@ int PctestIpv6Udp::GetSocketOut() {
 //
 // Output:
 //
-// A negative icmpCode indicates a timeout.
+// A negative icmpCode indicates an error.
 //
 int PctestIpv6Udp::Test(TestRecord &tr)
 {
@@ -115,6 +115,9 @@ int PctestIpv6Udp::Test(TestRecord &tr)
     fd_set readFds;		// reading file descriptors
     int done = 0;
 
+    // Parameters stored as globals
+    extern int Timeout;
+
     // If the requested sending size is too small, then return an
     // error.  The caller should have figured out the minimum sending
     // size by calling Pctest::GetMinSize().
@@ -140,8 +143,16 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	return rc;
     }
 
+    // Use malloc(3) to allocate (memory-aligned) space for the inbound
+    // packet.
+    char *icmp6PacketIn;
+    icmp6PacketIn = (char *) malloc(IPV6_MAXPACKET);
+    if (icmp6PacketIn == NULL) {
+	fprintf(stderr, "Couldn't allocate space for inbound packet\n");
+	return -1;
+    }
+
     // Set timeout value and socket select parameters
-    extern int Timeout;
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
@@ -154,10 +165,9 @@ int PctestIpv6Udp::Test(TestRecord &tr)
     rc = sendto(socketOut, udpPayload, udpPayloadSize, 0,
 		(struct sockaddr *) &targetSocketAddress,
 		sizeof(struct sockaddr_in6));
-    delete [] udpPayload;
     if (rc < 0) {
 	perror("sendto");
-	return rc;
+	goto exittest;
     }
 
     // We need to check the socket until we get a valid packet.
@@ -168,7 +178,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
 	    perror("select");
-	    return rc;
+	    goto exittest;
 	}
 
 	// Timestamp after and update test record timestamp fields
@@ -184,20 +194,17 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
-	    const int icmpPacketSize = 1500 + tr.size;
-	    char icmpPacket[IPV6_MAXPACKET];
 	    struct msghdr msg;	// msghdr is for recvmsg
 	    struct iovec iov[1];
 	    int controlsize;
-	    struct cmsghdr *cmsgptr;
 
 	    // Fill in the message header so we can read all the
 	    // metadata from the ICMP packet.  A lot harder than
 	    // with ICMPv4 since we had the IP header to work with.
 	    msg.msg_name = (char *) &icmpSourceSocketAddress;
 	    msg.msg_namelen = sizeof(icmpSourceSocketAddress);
-	    iov[0].iov_base = icmpPacket;
-	    iov[0].iov_len = icmpPacketSize;
+	    iov[0].iov_base = icmp6PacketIn;
+	    iov[0].iov_len = IPV6_MAXPACKET;
 	    msg.msg_iov = iov;
 	    msg.msg_iovlen = 1;
 
@@ -221,7 +228,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	    rc = recvmsg(socketIn, &msg, 0);
 	    if (rc < 0) {
 		perror("read");
-		return rc;
+		goto exittest;
 	    }
 	    tr.replsize = rc;
 
@@ -240,7 +247,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	    }
 
 	    // Find the ICMPv6 header
-	    icmpHeader = (icmp6_hdr *) icmpPacket;
+	    icmpHeader = (icmp6_hdr *) icmp6PacketIn;
 	    IF_DEBUG(3, fprintf(stderr, "ICMPv6 type = %d, code = %d\n", 
 				icmpHeader->icmp6_type, icmpHeader->icmp6_code));
 
@@ -333,7 +340,12 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 
     } while (!done);
 
-    return 0;
+    rc = 0;
+
+  exittest:
+    delete [] udpPayload;
+    free(icmp6PacketIn);
+    return rc;
 
 }
 
@@ -345,7 +357,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 // Output:  Minimum packet size possible for this protocol (in return
 // value).
 //
-int PctestIpv6Udp::GetMinSize() 
+unsigned int PctestIpv6Udp::GetMinSize() 
 {
     return (sizeof(ip6_hdr) + sizeof(udphdr) + 4);
 }
