@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6Udp.h,v 1.5 2001/01/02 18:46:15 bmah Exp $
+// $Id: PctestIpv6Udp.h,v 1.6 2001/02/01 21:38:59 bmah Exp $
 //
 // PctestIpv6Udp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -52,7 +52,7 @@ class PctestIpv6Udp : public PctestIpv6 {
 
     virtual int GetSocketOut();
     virtual int Test(TestRecord &tr);
-    virtual int GetMinSize();
+    virtual unsigned int GetMinSize();
     virtual PctestActionType GetAction(int icmpType, int icmpCode);
    
   protected:
