@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: ResultTable.cc,v 1.22 2001/01/02 21:21:08 bmah Exp $";
+static char rcsid[] = "$Id: ResultTable.cc,v 1.24 2001/02/28 23:02:48 bmah Exp $";
 //
-// $Id: ResultTable.cc,v 1.22 2001/01/02 21:21:08 bmah Exp $
+// $Id: ResultTable.cc,v 1.24 2001/02/28 23:02:48 bmah Exp $
 //
 // ResultTable.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -25,14 +25,15 @@ static char rcsid[] = "$Id: ResultTable.cc,v 1.22 2001/01/02 21:21:08 bmah Exp $
 //
 // Constructor
 //
-// Input:  Table parameters (i, m, r)
+// Input:  Table parameters (i, m, b, r)
 //
 // Output:  None
 //
-ResultTable::ResultTable(int inc, int m, int r) : 
-    increment(inc), mtu(m), repetitions(r), columns(2*mtu)
+ResultTable::ResultTable(unsigned int inc, unsigned int m, unsigned int b, 
+			 unsigned int r) : 
+    increment(inc), mtu(m), repetitions(r), burst(b), columns((burst+1)*mtu)
 // XXX Note the initialization of the columns member; we want to be
-// able to hold the largest possible response packet.  2*MTU should
+// able to hold the largest possible response packet.  (burst+1)*MTU should
 // work unless the user sets MTU to something small.  There should
 // be a better way to handle this.
 {
@@ -46,7 +47,7 @@ ResultTable::ResultTable(int inc, int m, int r) :
     data = new (double *[columns]);
     used = new int[columns];
     for (i = 0; i < columns; i++) {
-	data[i] = new double[repetitions];
+	data[i] = NULL;
 	used[i] = 0;
     }
 
@@ -68,7 +69,10 @@ ResultTable::~ResultTable() {
     int i;
 
     for (i = 0; i < columns; i++) {
-	delete [] data[i];
+	if (data[i]) {
+	    delete [] data[i];
+	    data[i] = NULL;
+	}
     }
     delete data;
     delete used;
@@ -97,6 +101,14 @@ int ResultTable::put(int size, double time) {
 	return -1;
     }
 
+    // Need to allocate more memory to hold this column?
+    if (data[offset] == NULL) {
+	data[offset] = new double[repetitions];
+	if (data[offset] == NULL) {
+	    return -1;
+	}
+    }
+
     // Store data
     data[offset][used[offset]++] = time;
     return 0;
@@ -121,7 +133,7 @@ ResultTable *ResultTable::getMin() {
 
     // Get new ResultTable, but we only need room for one
     // "repetition".
-    ResultTable *t2 = new ResultTable(increment, mtu, 1);
+    ResultTable *t2 = new ResultTable(increment, mtu, burst, 1);
     if (t2 == NULL) {
 	return NULL;
     }
@@ -140,11 +152,9 @@ ResultTable *ResultTable::getMin() {
 		}
 	    }
 
-	    // Store the data directly into the new structure.
-	    // It's probably evil to do this without using the
-	    // access method put().
-	    t2->data[i][0] = min;
-	    t2->used[i]++;
+	    if (t2->put(column2size(i), min) < 0) {
+		return NULL;
+	    }
 	}
     }
     return t2;
