@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: main.cc,v 1.83 2001/01/27 00:27:08 bmah Exp $";
+static char rcsid[] = "$Id: main.cc,v 1.96 2001/04/03 19:41:35 bmah Exp $";
 //
-// $Id: main.cc,v 1.83 2001/01/27 00:27:08 bmah Exp $
+// $Id: main.cc,v 1.96 2001/04/03 19:41:35 bmah Exp $
 //
 // main.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -76,7 +76,8 @@ void PrintIfInfo(const GetIfInfo *);
 // Default values
 //
 AnalysisType Analysis = AnalysisLeastSquares;
-int ChangeFlag = 0;
+unsigned int Burst = 1;
+bool ChangeFlag = false;
 int DebugLevel = 0;
 double Gap = 0.25;
 GapType GapDist = GapFixed;
@@ -106,6 +107,7 @@ char *TargetHost = NULL;
 //
 ResultTable **PartialResults;
 ResultTable **PartialMins;
+const int MaxAddressesPerHop = 10;
 const int MaxSocketAddressLength = 255;
 
 //
@@ -147,7 +149,7 @@ void Usage(char *argv0) {
     fprintf(stderr, "\t\t\tkendall\tLinear fit using Kendall's test statistic\n");
     fprintf(stderr, "\t\t\tlms\tLeast median of squares linear fit\n");
     fprintf(stderr, "\t\t\tlmsint\tLeast median of squares linear fit (integer computations)\n");
-    fprintf(stderr, "\t-c\t\tIgnore route changes (not recommended)\n");
+    fprintf(stderr, "\t-c\t\tIgnore route changes\n");
     fprintf(stderr, "\t-d debuglevel\tSet debugging output level\n");
     fprintf(stderr, "\t-g gap\t\tInter-test gap in seconds (default = %0.2f)\n", Gap);
     fprintf(stderr, "\t-G gaptype\tInter-test gap type (default is fixed)\n");
@@ -303,7 +305,7 @@ char *GetPrintableNetworkProtocol(NetworkProtocolType np)
 //
 // main
 //
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 
     int c;			// getopt
@@ -339,7 +341,7 @@ main(int argc, char **argv)
 
 	// c: ignore route changes
 	case 'c': {
-	    ChangeFlag = 1;
+	    ChangeFlag = true;
 	    break;
 	}
 
@@ -407,6 +409,10 @@ main(int argc, char **argv)
 	// m: MTU size
 	case 'm': {
 	    Mtu = atoi(optarg);
+	    if (!Mtu) {
+		fprintf(stderr, "Mtu argument must be positive (was %d)\n", Mtu);
+		exit(1);
+	    }
 	    break;
 	}
 
@@ -477,13 +483,11 @@ main(int argc, char **argv)
 
 	// r: read from file
 	case 'r': {
-	    int len = strlen(optarg);
-	    ReadFilename = (char *) malloc(len + 1);
+	    ReadFilename = strdup(optarg);
 	    if (ReadFilename == NULL) {
 		fprintf(stderr, "Couldn't allocate space for statistics filename\n");
 		exit(1);
 	    }
-	    strncpy(ReadFilename, optarg, len + 1);
 	    if (WriteFilename) {
 		fprintf(stderr, "Warning: both -r and -w specified\n");
 	    }
@@ -549,13 +553,11 @@ main(int argc, char **argv)
 
 	// w: write statistics to file
 	case 'w': {
-	    int len = strlen(optarg);
-	    WriteFilename = (char *) malloc(len + 1);
+	    WriteFilename = strdup(optarg);
 	    if (WriteFilename == NULL) {
 		fprintf(stderr, "Couldn't allocate space for statistics filename\n");
 		exit(1);
 	    }
-	    strncpy(WriteFilename, optarg, len + 1);
 	    if (ReadFilename) {
 		fprintf(stderr, "Warning: both -r and -w specified\n");
 	    }
@@ -600,6 +602,14 @@ main(int argc, char **argv)
 	
 	char *af;
 
+#ifdef WITH_SUID
+	// If we're running SUID root, then we drop privileges
+	// here.  We don't want to read random files in the
+	// filesystem, and the main reason we needed superuser 
+	// in the first place (raw sockets) doesn't apply.
+	setuid(getuid());
+#endif /* WITH_SUID */
+
 	// We need to read the file first to figure out what address
 	// family it describes.
 	af = GetAddressFamily();
@@ -742,6 +752,15 @@ main(int argc, char **argv)
 	exit(1);
     }
 
+#ifdef WITH_SUID
+    // If we were running SUID root, then drop privileges here
+    // (assuming we didn't do this already).  It's not so great to
+    // run this far into the program with superuser privileges.
+    // At least we give them up in any case before we write to
+    // output files.
+    setuid(getuid());
+#endif /* WITH_SUID */
+
     // With all arguments parsed, determine which set of tests
     // to run and go do it.
     switch (Mode) {
@@ -775,7 +794,7 @@ main(int argc, char **argv)
 //
 void DoPchar(Pctest *pct) 
 {
-    int i, j, k;		// universal loop counters
+    int i, j, k, l;		// universal loop counters
     FILE *df = NULL;		// output file 
 
     // Generate set of packet sizes to test.  We'll test packets
@@ -838,6 +857,7 @@ void DoPchar(Pctest *pct)
 	
 	timeouts = 0;
 	
+	tr->burst = 1;		// only need one packet for pings
 	tr->size = 128;		// what packet size is best here?
 	if (tr->size < pct->GetMinSize()) {
 	    tr->size = pct->GetMinSize();
@@ -920,6 +940,7 @@ void DoPchar(Pctest *pct)
 	fprintf(df, "minsize %d\n", pct->GetMinSize());
 	fprintf(df, "increment %d\n", Increment);
 	fprintf(df, "mtu %d\n", Mtu);
+	fprintf(df, "burst %d\n", Burst);
 	fprintf(df, "repetitions %d\n", Repetitions);
 	fprintf(df, "starthop %d\n", StartHop);
     }
@@ -930,21 +951,32 @@ void DoPchar(Pctest *pct)
     // keep the hop number minus StartHop, and add it back before printing,
     // since the internal data structures we use start from zero.)
     //
-    bool lastHopFlag;		// *could* be the last hop
+    bool firstProbe = true;	// first probe flag
+    struct timeval timeFirst, timeLast;	// time of first and last probes
+    bool lastHopFlag = true;	// *could* be the last hop
     double aCumulativeLast = 0.0,
 	   bCumulativeLast = 0.0,
 	   r2CumulativeLast = 0.0;
     double minBandwidth = -1.0; // minimum bandwidth found so far
     double queueingTime = 0.0;	// total queueing time seen so far
     int queueingBytes = 0;	// total queued bytes estimated so far
+
+    char *checkAddress[MaxAddressesPerHop];
+    int checkAddressLength[MaxAddressesPerHop];
+    for (i = 0; i < MaxAddressesPerHop; i++) {
+	checkAddress[i] = new char[MaxSocketAddressLength];
+	if (checkAddress[i] == NULL) {
+	    fprintf(stderr, "Couldn't allocate space for route change detection\n");
+	    exit(1);
+	}
+    }
+
     for (i = 0; i < Hops; i++) {
 
 	int packetsLost = 0;
 	int packetsSent = 0;
 
-	char *checkAddress = new char[MaxSocketAddressLength]; // for detecting
-	int checkAddressLength = 0; // routing changes
-
+	unsigned addressesSeen = 0; // for detecting routing changes
 	bool realTimestamp = false;	// set true once we've received 
 				// real timestamp that didn't need
 				// adjustments
@@ -952,7 +984,8 @@ void DoPchar(Pctest *pct)
 	PctestActionType pcta;
 
 	// Initialize statistics structure
-	PartialResults[i] = new ResultTable(Increment, Mtu, Repetitions);
+	PartialResults[i] = new ResultTable(Increment, Mtu, Burst, 
+					    Repetitions);
 	lastHopFlag = true;
 
 	// Run the correct number of repetitions and iterate over the
@@ -972,6 +1005,7 @@ void DoPchar(Pctest *pct)
 
 		// Set up variables for a test
 		tr = new TestRecord;
+		tr->burst = Burst;
 		tr->size = packetSize[k];
 		if (tr->size < pct->GetMinSize()) {
 		    // Skip this test if the packet size is too small
@@ -988,6 +1022,15 @@ void DoPchar(Pctest *pct)
 		    exit(1);
 		}
 
+		// Update start and end timestamps for this session
+		if (firstProbe) {
+		    timeFirst.tv_sec = tr->tvstart.tv_sec;
+		    timeFirst.tv_usec = tr->tvstart.tv_usec;
+		    firstProbe = false;
+		}
+		timeLast.tv_sec = tr->tvstart.tv_sec;
+		timeLast.tv_usec = tr->tvstart.tv_usec;
+
 		// Write to disk if requested
 		if (WriteFilename) {
 		    fprintf(df, "%s\n", tr->htoa(pct));
@@ -1017,24 +1060,33 @@ void DoPchar(Pctest *pct)
 
 		IF_DEBUG(2, fprintf(stderr, "bytes = %d, rtt = %f, ip_src = %s, replbytes = %d\n", tr->size, rtt, pct->GetPrintableAddress(tr->icmpSourceAddress), tr->replsize));
 
-		// If address changed, we're in trouble because the route
-		// changed.
-		if (checkAddressLength == 0) {
-		    memcpy(checkAddress, tr->icmpSourceAddress, tr->icmpSourceAddressLength);
-		    checkAddressLength = tr->icmpSourceAddressLength;
+		// See if we've seen this address before or not.  If so, 
+		// record it for posterity.
+		for (l = 0; l < addressesSeen; l++) {
+		    if ((checkAddressLength[l] == 
+			 tr->icmpSourceAddressLength) && 
+			(memcmp(tr->icmpSourceAddress, checkAddress[l], 
+				checkAddressLength[l]) == 0)) {
+			break;
+		    }
 		}
-		else if ((tr->icmpSourceAddressLength != checkAddressLength) ||
-			 (memcmp(tr->icmpSourceAddress, checkAddress, checkAddressLength) != 0)) {
-
-		    fprintf(stderr, "Route change detected (");
-		    fprintf(stderr, "%s != ", pct->GetPrintableAddress(tr->icmpSourceAddress));
-		    fprintf(stderr, "%s)\n", pct->GetPrintableAddress(checkAddress));
-		    
-		    if (!ChangeFlag) {
-			// We're not ignoring route changes, so die here.
-			exit(1);
+		if (l == addressesSeen) {
+		    if (addressesSeen == 1) {
+			if (!QuietFlag) {
+			    fprintf(stdout, "Route change detected\n");
+			}
+			if ((!ChangeFlag) && (ReadFilename)) {
+			    exit(1);
+			}
+		    }
+		    if (addressesSeen < MaxAddressesPerHop) {
+			memcpy(checkAddress[addressesSeen],
+			       tr->icmpSourceAddress,
+			       tr->icmpSourceAddressLength);
+			checkAddressLength[addressesSeen] = 
+			    tr->icmpSourceAddressLength;
+			addressesSeen++;
 		    }
-
 		}
 
 		// Have the Pctest subclass figure out what the ICMP
@@ -1047,8 +1099,8 @@ void DoPchar(Pctest *pct)
 		if ((pcta == PctestActionValid) || 
 		    (pcta == PctestActionValidLasthop)) {
 
-		    if ((PartialResults[i]->put(tr->size + tr->replsize, 
-						rtt)) < 0) {
+		    if ((PartialResults[i]->put(
+			(tr->burst * tr->size) + tr->replsize, rtt)) < 0) {
 			fprintf(stderr, "Couldn't store result\n");
 			abort();
 		    }
@@ -1146,7 +1198,7 @@ endreps:
 	// Per-hop processing and statistics.
 	PartialMins[i] = PartialResults[i]->getMin();
 	
-	double aCumulative, bCumulative, r2Cumulative, aHop, bHop, r2Hop;
+	double aCumulative, bCumulative, r2Cumulative, aHop, bHop;
 	double sa, sb;
 	double bLower, bUpper;
 
@@ -1213,6 +1265,7 @@ endreps:
 
 	// Per-hop output
 	if (!QuietFlag) {
+
 	    fprintf(stdout, "    Partial loss:      %d / %d (%d%%)\n", packetsLost, packetsSent, packetsLost * 100 / packetsSent);
 
 	    if (Analysis == AnalysisLeastSquares) {
@@ -1259,15 +1312,22 @@ endreps:
 
 	    }
 
-	    fprintf(stdout, "%2d: %s ", i+StartHop, 
-		    pct->GetPrintableAddress(checkAddress));
-	    if (NumericFlag) {
-		fprintf(stdout, "(%s)", pct->GetPrintableAddress(checkAddress));
+	    if (addressesSeen > 0) {
+		for (l = 0; l < addressesSeen; l++) {
+		    fprintf(stdout, "%2d: %s ", i+StartHop, 
+			    pct->GetPrintableAddress(checkAddress[l]));
+		    if (NumericFlag) {
+			fprintf(stdout, "(%s)\n", pct->GetPrintableAddress(checkAddress[l]));
+		    }
+		    else {
+			fprintf(stdout, "(%s)\n", pct->GetName(checkAddress[l]));
+		    }
+		}
 	    }
 	    else {
-		fprintf(stdout, "(%s)", pct->GetName(checkAddress));
+		fprintf(stderr, "Received probe responses but no addresses recorded\n");
+		abort();
 	    }
-	    fprintf(stdout, "\n");
 
 #ifdef HAVE_SNMP
 	    if (!ReadFilename && SnmpFlag) {
@@ -1351,8 +1411,18 @@ endreps:
 	else {
 	    fprintf(stdout, "    End of path not reached after %d hops\n", pathLength);
 	}
+
+	fprintf(stdout, "    Start time:        %s", ctime(&(timeFirst.tv_sec)));
+	fprintf(stdout, "    End time:          %s", ctime(&(timeLast.tv_sec)));
     }
 
+    //
+    // Free memory, close files, etc.
+    //
+    for (i = 0; i < MaxAddressesPerHop; i++) {
+	delete [] checkAddress[i];
+	checkAddress[i] = NULL;
+    }
     if (WriteFilename) {
 	fclose(df);
     }
@@ -1472,9 +1542,13 @@ void DoTrout(Pctest *pct)
 
 	// Set up variables for a test
 	tr = new TestRecord;
+	tr->burst = 1;
 	tr->size = packetSize;
 	tr->hops = i+StartHop;
 	gettimeofday(&tr->tvstart, NULL);
+	tr->tv.tv_sec = 255;
+	tr->tv.tv_usec = 0;
+	tr->replsize = 0;
 	
 	rc = pct->Test(*tr);
 	packetsSent++;
@@ -1616,8 +1690,6 @@ endreps:
 	}
     }
     
-    int pathLength = i + StartHop;
-
     // End-of-run processing
     if (!QuietFlag) {
     }
